"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DispatcherClient = void 0;
const dispatcher_1 = require("../../protoc/dispatcher");
const hatchet_error_1 = __importDefault(require("../../util/errors/hatchet-error"));
const retrier_1 = require("../../util/retrier");
const version_1 = require("../../version");
const action_listener_1 = require("./action-listener");
class DispatcherClient {
    constructor(config, channel, factory) {
        this.config = config;
        this.client = factory.create(dispatcher_1.DispatcherDefinition, channel);
        this.logger = config.logger(`Dispatcher`, config.log_level);
    }
    getRuntimeInfo() {
        return {
            sdkVersion: version_1.HATCHET_VERSION,
            language: dispatcher_1.SDKS.TYPESCRIPT,
            languageVersion: process.version,
            os: process.platform,
        };
    }
    getActionListener(options) {
        return __awaiter(this, void 0, void 0, function* () {
            // Register the worker
            const registration = yield this.client.register(Object.assign(Object.assign({}, options), { labels: options.labels ? mapLabels(options.labels) : undefined, runtimeInfo: this.getRuntimeInfo() }));
            return new action_listener_1.ActionListener(this, registration.workerId);
        });
    }
    sendStepActionEvent(in_) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield (0, retrier_1.retrier)(() => __awaiter(this, void 0, void 0, function* () { return this.client.sendStepActionEvent(in_); }), this.logger);
            }
            catch (e) {
                throw new hatchet_error_1.default(e.message);
            }
        });
    }
    sendGroupKeyActionEvent(in_) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield (0, retrier_1.retrier)(() => __awaiter(this, void 0, void 0, function* () { return this.client.sendGroupKeyActionEvent(in_); }), this.logger);
            }
            catch (e) {
                throw new hatchet_error_1.default(e.message);
            }
        });
    }
    putOverridesData(in_) {
        return __awaiter(this, void 0, void 0, function* () {
            return (0, retrier_1.retrier)(() => __awaiter(this, void 0, void 0, function* () { return this.client.putOverridesData(in_); }), this.logger).catch((e) => {
                this.logger.warn(`Could not put overrides data: ${e.message}`);
            });
        });
    }
    refreshTimeout(incrementTimeoutBy, stepRunId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return this.client.refreshTimeout({
                    stepRunId,
                    incrementTimeoutBy,
                });
            }
            catch (e) {
                throw new hatchet_error_1.default(e.message);
            }
        });
    }
    upsertWorkerLabels(workerId, labels) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.client.upsertWorkerLabels({
                    workerId,
                    labels: mapLabels(labels),
                });
            }
            catch (e) {
                throw new hatchet_error_1.default(e.message);
            }
        });
    }
}
exports.DispatcherClient = DispatcherClient;
function mapLabels(in_) {
    return Object.entries(in_).reduce((acc, [key, value]) => (Object.assign(Object.assign({}, acc), { [key]: {
            strValue: typeof value === 'string' ? value : undefined,
            intValue: typeof value === 'number' ? value : undefined,
        } })), {});
}
