import { ChannelCredentials } from 'nice-grpc';
import { z } from 'zod';
import { Logger, LogLevel } from '../../util/logger';
declare const ClientTLSConfigSchema: z.ZodObject<{
    tls_strategy: z.ZodOptional<z.ZodEnum<["tls", "mtls", "none"]>>;
    cert_file: z.ZodOptional<z.ZodString>;
    ca_file: z.ZodOptional<z.ZodString>;
    key_file: z.ZodOptional<z.ZodString>;
    server_name: z.ZodOptional<z.ZodString>;
}, "strip", z.ZodTypeAny, {
    tls_strategy?: "tls" | "mtls" | "none" | undefined;
    cert_file?: string | undefined;
    ca_file?: string | undefined;
    key_file?: string | undefined;
    server_name?: string | undefined;
}, {
    tls_strategy?: "tls" | "mtls" | "none" | undefined;
    cert_file?: string | undefined;
    ca_file?: string | undefined;
    key_file?: string | undefined;
    server_name?: string | undefined;
}>;
export declare const ClientConfigSchema: z.ZodObject<{
    token: z.ZodString;
    tls_config: z.ZodObject<{
        tls_strategy: z.ZodOptional<z.ZodEnum<["tls", "mtls", "none"]>>;
        cert_file: z.ZodOptional<z.ZodString>;
        ca_file: z.ZodOptional<z.ZodString>;
        key_file: z.ZodOptional<z.ZodString>;
        server_name: z.ZodOptional<z.ZodString>;
    }, "strip", z.ZodTypeAny, {
        tls_strategy?: "tls" | "mtls" | "none" | undefined;
        cert_file?: string | undefined;
        ca_file?: string | undefined;
        key_file?: string | undefined;
        server_name?: string | undefined;
    }, {
        tls_strategy?: "tls" | "mtls" | "none" | undefined;
        cert_file?: string | undefined;
        ca_file?: string | undefined;
        key_file?: string | undefined;
        server_name?: string | undefined;
    }>;
    host_port: z.ZodString;
    api_url: z.ZodString;
    log_level: z.ZodOptional<z.ZodEnum<["OFF", "DEBUG", "INFO", "WARN", "ERROR"]>>;
    tenant_id: z.ZodString;
    namespace: z.ZodOptional<z.ZodString>;
    grpc_max_send_message_length: z.ZodOptional<z.ZodNumber>;
    grpc_max_recv_message_length: z.ZodOptional<z.ZodNumber>;
}, "strip", z.ZodTypeAny, {
    token: string;
    tls_config: {
        tls_strategy?: "tls" | "mtls" | "none" | undefined;
        cert_file?: string | undefined;
        ca_file?: string | undefined;
        key_file?: string | undefined;
        server_name?: string | undefined;
    };
    host_port: string;
    api_url: string;
    tenant_id: string;
    log_level?: "OFF" | "DEBUG" | "INFO" | "WARN" | "ERROR" | undefined;
    namespace?: string | undefined;
    grpc_max_send_message_length?: number | undefined;
    grpc_max_recv_message_length?: number | undefined;
}, {
    token: string;
    tls_config: {
        tls_strategy?: "tls" | "mtls" | "none" | undefined;
        cert_file?: string | undefined;
        ca_file?: string | undefined;
        key_file?: string | undefined;
        server_name?: string | undefined;
    };
    host_port: string;
    api_url: string;
    tenant_id: string;
    log_level?: "OFF" | "DEBUG" | "INFO" | "WARN" | "ERROR" | undefined;
    namespace?: string | undefined;
    grpc_max_send_message_length?: number | undefined;
    grpc_max_recv_message_length?: number | undefined;
}>;
export type LogConstructor = (context: string, logLevel?: LogLevel) => Logger;
export type ClientConfig = z.infer<typeof ClientConfigSchema> & {
    credentials?: ChannelCredentials;
} & {
    logger: LogConstructor;
};
export type ClientTLSConfig = z.infer<typeof ClientTLSConfigSchema>;
export {};
