"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HatchetLogger = exports.DEFAULT_LOGGER = void 0;
/* eslint-disable no-console */
const logger_1 = require("../../util/logger");
const DEFAULT_LOGGER = (context, logLevel) => new HatchetLogger(context, logLevel);
exports.DEFAULT_LOGGER = DEFAULT_LOGGER;
class HatchetLogger {
    constructor(context, logLevel = 'INFO') {
        this.logLevel = logLevel;
        this.context = context;
    }
    log(level, message, color = '37') {
        if (logger_1.LogLevelEnum[level] >= logger_1.LogLevelEnum[this.logLevel]) {
            const time = new Date().toLocaleString('en-US', {
                month: '2-digit',
                day: '2-digit',
                year: '2-digit',
                hour: '2-digit',
                minute: '2-digit',
                second: '2-digit',
            });
            // eslint-disable-next-line prefer-destructuring
            let print = console.log;
            if (level === 'ERROR') {
                print = console.error;
            }
            if (level === 'WARN') {
                print = console.warn;
            }
            if (level === 'INFO') {
                print = console.info;
            }
            if (level === 'DEBUG') {
                print = console.debug;
            }
            // eslint-disable-next-line no-console
            print(`🪓 ${process.pid} | ${time} ${color && `\x1b[${color || ''}m`} [${level}/${this.context}] ${message}\x1b[0m`);
        }
    }
    debug(message) {
        this.log('DEBUG', message, '35');
    }
    info(message) {
        this.log('INFO', message);
    }
    green(message) {
        this.log('INFO', message, '32');
    }
    warn(message, error) {
        this.log('WARN', `${message} ${error}`, '93');
    }
    error(message, error) {
        this.log('ERROR', `${message} ${error}`, '91');
    }
}
exports.HatchetLogger = HatchetLogger;
