import { InternalHatchetClient } from '../hatchet-client';
import { Action, ActionListener } from '../dispatcher/action-listener';
import { StepActionEvent, StepActionEventType, GroupKeyActionEvent, GroupKeyActionEventType } from '../../protoc/dispatcher';
import HatchetPromise from '../../util/hatchet-promise/hatchet-promise';
import { Workflow } from '../../workflow';
import { Logger } from '../../util/logger';
import { WebhookHandler } from './handler';
import { WebhookWorkerCreateRequest } from '../rest/generated/data-contracts';
import { BaseWorkflowDeclaration, WorkflowDefinition } from '../../v1/declaration';
import { Context, StepRunFunction } from '../../step';
import { WorkerLabels } from '../dispatcher/dispatcher-client';
export type ActionRegistry = Record<Action['actionId'], Function>;
export interface WorkerOpts {
    name: string;
    handleKill?: boolean;
    maxRuns?: number;
    labels?: WorkerLabels;
}
export declare class V0Worker {
    client: InternalHatchetClient;
    name: string;
    workerId: string | undefined;
    killing: boolean;
    handle_kill: boolean;
    action_registry: ActionRegistry;
    workflow_registry: Array<WorkflowDefinition | Workflow>;
    listener: ActionListener | undefined;
    futures: Record<Action['stepRunId'], HatchetPromise<any>>;
    contexts: Record<Action['stepRunId'], Context<any, any>>;
    maxRuns?: number;
    logger: Logger;
    registeredWorkflowPromises: Array<Promise<any>>;
    labels: WorkerLabels;
    constructor(client: InternalHatchetClient, options: {
        name: string;
        handleKill?: boolean;
        maxRuns?: number;
        labels?: WorkerLabels;
    });
    private registerActions;
    getHandler(workflows: Workflow[]): WebhookHandler;
    registerWebhook(webhook: WebhookWorkerCreateRequest): Promise<import("axios").AxiosResponse<import("../rest/generated/data-contracts").WebhookWorkerCreated, any>>;
    /**
     * @deprecated use registerWorkflow instead
     */
    register_workflow(initWorkflow: Workflow): Promise<void>;
    registerDurableActionsV1(workflow: WorkflowDefinition): void;
    private registerActionsV1;
    registerWorkflowV1(initWorkflow: BaseWorkflowDeclaration<any, any>): Promise<void>;
    registerWorkflow(initWorkflow: Workflow): Promise<void>;
    registerAction<T, K>(actionId: string, action: StepRunFunction<T, K>): void;
    handleStartStepRun(action: Action): Promise<void>;
    handleStartGroupKeyRun(action: Action): Promise<void>;
    getStepActionEvent(action: Action, eventType: StepActionEventType, shouldNotRetry: boolean, payload?: any, retryCount?: number): StepActionEvent;
    getGroupKeyActionEvent(action: Action, eventType: GroupKeyActionEventType, payload?: any): GroupKeyActionEvent;
    handleCancelStepRun(action: Action): Promise<void>;
    stop(): Promise<void>;
    exitGracefully(handleKill: boolean): Promise<void>;
    start(): Promise<void>;
    handleAction(action: Action): Promise<void>;
    upsertLabels(labels: WorkerLabels): Promise<WorkerLabels>;
}
