"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// eslint-disable-next-line import/no-extraneous-dependencies
const pino_1 = __importDefault(require("pino"));
const sdk_1 = __importDefault(require("../sdk"));
// ❓ Create Pino logger
const logger = (0, pino_1.default)();
class PinoLogger {
    constructor(context, logLevel = 'DEBUG') {
        this.logLevel = logLevel;
        this.context = context;
    }
    debug(message) {
        logger.debug(message);
    }
    info(message) {
        logger.info(message);
    }
    green(message) {
        logger.info(`%c${message}`);
    }
    warn(message, error) {
        logger.warn(`${message} ${error}`);
    }
    error(message, error) {
        logger.error(`${message} ${error}`);
    }
}
const hatchet = sdk_1.default.init({
    log_level: 'DEBUG',
    logger: (ctx, level) => new PinoLogger(ctx, level),
});
// !!
// ❓ Use the logger
const sleep = (ms) => new Promise((resolve) => {
    setTimeout(resolve, ms);
});
const workflow = {
    id: 'byo-logger-example',
    description: 'An example showing how to pass a custom logger to Hatchet',
    on: {
        event: 'byo-logger:spawn',
    },
    steps: [
        {
            name: 'logger-step1',
            run: (ctx) => __awaiter(void 0, void 0, void 0, function* () {
                // eslint-disable-next-line no-plusplus
                for (let i = 0; i < 5; i++) {
                    logger.info(`log message ${i}`);
                    yield sleep(500);
                }
                return { step1: 'completed step run' };
            }),
        },
    ],
};
// !!
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const worker = yield hatchet.worker('byo-logger-worker', 1);
        yield worker.registerWorkflow(workflow);
        worker.start();
    });
}
main();
