import WorkflowRunRef from '../../../util/workflow-run-ref';
import { V1TaskStatus } from '../../../clients/rest/generated/data-contracts';
import { WorkflowsClient } from './workflows';
import { HatchetClient } from '../client';
export type RunFilter = {
    since?: Date;
    until?: Date;
    statuses?: V1TaskStatus[];
    workflowNames?: string[];
    additionalMetadata?: Record<string, string>;
};
export type CancelRunOpts = {
    ids?: string[];
    filters?: RunFilter;
};
export type ReplayRunOpts = {
    ids?: string[];
    filters?: RunFilter;
};
export interface ListRunsOpts extends RunFilter {
    /**
     * The number to skip
     * @format int64
     */
    offset?: number;
    /**
     * The number to limit by
     * @format int64
     */
    limit?: number;
    /** A list of statuses to filter by */
    /**
     * The worker id to filter by
     * @format uuid
     * @minLength 36
     * @maxLength 36
     */
    workerId?: string;
    /** Whether to include DAGs or only to include tasks */
    onlyTasks: boolean;
    /**
     * The parent task external id to filter by
     * @format uuid
     * @minLength 36
     * @maxLength 36
     */
    parentTaskExternalId?: string;
}
/**
 * RunsClient is used to list and manage runs
 */
export declare class RunsClient {
    api: HatchetClient['api'];
    tenantId: string;
    workflows: WorkflowsClient;
    constructor(client: HatchetClient);
    get<T = any>(run: string | WorkflowRunRef<T>): Promise<import("../../../clients/rest/generated/data-contracts").V1WorkflowRunDetails>;
    list(opts?: Partial<ListRunsOpts>): Promise<import("../../../clients/rest/generated/data-contracts").V1TaskSummaryList>;
    cancel(opts: CancelRunOpts): Promise<import("axios").AxiosResponse<void, any>>;
    replay(opts: ReplayRunOpts): Promise<import("axios").AxiosResponse<void, any>>;
    private prepareFilter;
    private prepareListFilter;
}
