"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunsClient = void 0;
/**
 * RunsClient is used to list and manage runs
 */
class RunsClient {
    constructor(client) {
        this.api = client.api;
        this.tenantId = client.tenantId;
        this.workflows = client.workflows;
    }
    get(run) {
        return __awaiter(this, void 0, void 0, function* () {
            const runId = typeof run === 'string' ? run : yield run.getWorkflowRunId();
            const { data } = yield this.api.v1WorkflowRunGet(runId);
            return data;
        });
    }
    list(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            const { data } = yield this.api.v1WorkflowRunList(this.tenantId, Object.assign({}, (yield this.prepareListFilter(opts || {}))));
            return data;
        });
    }
    cancel(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            const filter = yield this.prepareFilter(opts.filters || {});
            return this.api.v1TaskCancel(this.tenantId, {
                externalIds: opts.ids,
                filter: !opts.ids ? filter : undefined,
            });
        });
    }
    replay(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            const filter = yield this.prepareFilter(opts.filters || {});
            return this.api.v1TaskReplay(this.tenantId, {
                externalIds: opts.ids,
                filter: !opts.ids ? filter : undefined,
            });
        });
    }
    prepareFilter(_a) {
        return __awaiter(this, arguments, void 0, function* ({ since, until, statuses, workflowNames, additionalMetadata, }) {
            const am = Object.entries(additionalMetadata || {}).map(([key, value]) => `${key}:${value}`);
            return {
                // default to 1 hour ago
                since: since ? since.toISOString() : new Date(Date.now() - 1000 * 60 * 60).toISOString(),
                until: until === null || until === void 0 ? void 0 : until.toISOString(),
                statuses,
                workflowIds: yield Promise.all((workflowNames === null || workflowNames === void 0 ? void 0 : workflowNames.map((name) => __awaiter(this, void 0, void 0, function* () { return (yield this.workflows.get(name)).metadata.id; }))) || []),
                additionalMetadata: am,
            };
        });
    }
    prepareListFilter(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            const am = Object.entries(opts.additionalMetadata || {}).map(([key, value]) => `${key}:${value}`);
            return {
                // default to 1 hour ago
                since: opts.since
                    ? opts.since.toISOString()
                    : new Date(Date.now() - 1000 * 60 * 60).toISOString(),
                until: (_a = opts.until) === null || _a === void 0 ? void 0 : _a.toISOString(),
                statuses: opts.statuses,
                worker_id: opts.workerId,
                workflow_ids: yield Promise.all(((_b = opts.workflowNames) === null || _b === void 0 ? void 0 : _b.map((name) => __awaiter(this, void 0, void 0, function* () { return (yield this.workflows.get(name)).metadata.id; }))) || []),
                additional_metadata: am,
                only_tasks: opts.onlyTasks || false,
            };
        });
    }
}
exports.RunsClient = RunsClient;
