"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkersClient = void 0;
/**
 * WorkersClient is used to list and manage workers
 */
class WorkersClient {
    constructor(client) {
        this.api = client.api;
        this.tenantId = client.tenantId;
    }
    get(workerId) {
        return __awaiter(this, void 0, void 0, function* () {
            const { data } = yield this.api.workerGet(workerId);
            return data;
        });
    }
    list() {
        return __awaiter(this, void 0, void 0, function* () {
            const { data } = yield this.api.workerList(this.tenantId);
            return data;
        });
    }
    isPaused(workerId) {
        return __awaiter(this, void 0, void 0, function* () {
            const wf = yield this.get(workerId);
            return wf.status === 'PAUSED';
        });
    }
    pause(workerId) {
        return __awaiter(this, void 0, void 0, function* () {
            const { data } = yield this.api.workerUpdate(workerId, {
                isPaused: true,
            });
            return data;
        });
    }
    unpause(workerId) {
        return __awaiter(this, void 0, void 0, function* () {
            const { data } = yield this.api.workerUpdate(workerId, {
                isPaused: false,
            });
            return data;
        });
    }
}
exports.WorkersClient = WorkersClient;
