"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskConditionsToPb = taskConditionsToPb;
exports.conditionsToPb = conditionsToPb;
const _1 = require(".");
const base_1 = require("./base");
const parent_condition_1 = require("./parent-condition");
function taskConditionsToPb(task) {
    const waitForConditions = (0, _1.Render)(base_1.Action.QUEUE, task.waitFor);
    const cancelIfConditions = (0, _1.Render)(base_1.Action.CANCEL, task.cancelIf);
    const skipIfConditions = (0, _1.Render)(base_1.Action.SKIP, task.skipIf);
    const mergedConditions = [...waitForConditions, ...cancelIfConditions, ...skipIfConditions];
    return conditionsToPb(mergedConditions);
}
function conditionsToPb(conditions) {
    const parentOverrideConditions = [];
    const sleepConditions = [];
    const userEventConditions = [];
    conditions.forEach((condition) => {
        if (condition instanceof _1.SleepCondition) {
            sleepConditions.push({
                base: baseToPb(condition.base),
                sleepFor: condition.sleepFor,
            });
        }
        else if (condition instanceof _1.UserEventCondition) {
            userEventConditions.push({
                base: Object.assign(Object.assign({}, baseToPb(condition.base)), { expression: condition.expression || '' }),
                userEventKey: condition.eventKey,
            });
        }
        else if (condition instanceof parent_condition_1.ParentCondition) {
            parentOverrideConditions.push({
                base: baseToPb(condition.base),
                parentReadableId: condition.parent.name,
            });
        }
    });
    return {
        parentOverrideConditions,
        sleepConditions,
        userEventConditions,
    };
}
function baseToPb(base) {
    return {
        readableDataKey: base.readableDataKey || '',
        action: base.action,
        orGroupId: base.orGroupId || (0, _1.generateGroupId)(),
        expression: base.expression || '',
    };
}
